package com.unissoft.webConfig.service;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.unissoft.MyConstant;
import com.unissoft.exception.StudentException;
import com.unissoft.result.ResultView;
import com.unissoft.utils.RedisUtil;
import com.unissoft.webConfig.service.impl.AccessLimit;
import com.unissoft.webConfig.service.impl.TokenService;
import org.apache.commons.lang3.text.StrBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * 文件描述
 *
 * @author tongxi.xia
 * @date 2021年02月05日 10:17
 */
@Service
public class TokenServiceImpl implements TokenService {

    Logger log = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private RedisUtil redisService;

    /**
     * 创建token
     *
     * @return
     */
    @Override
    public String createToken() {
        String str = RandomUtil.randomUUID();
        StrBuilder token = new StrBuilder();
        try {
            token.append(MyConstant.TOKEN_PREFIX).append(str);
            redisService.set(token.toString(), token.toString(), MyConstant.TOKEN_OUT_TIME);
            boolean notEmpty = StrUtil.isNotEmpty(token.toString());
            if (notEmpty) {
                return token.toString();
            }
        } catch (Exception ex) {
            log.error(MyConstant.CREATE_TOKEN_FAIL);
        }
        return null;
    }

    /**
     * 检验token
     *
     * @param request
     * @return
     */
    @Override
    public boolean checkToken(HttpServletRequest request) throws StudentException {

        String token = request.getHeader(MyConstant.TOKEN_NAME);
        if (StrUtil.isBlank(token)) {
            // header中不存在token
            token = request.getParameter(MyConstant.TOKEN_NAME);
            if (StrUtil.isBlank(token)) {
                // parameter中也不存在token
                throw new StudentException(MyConstant.TOKEN_CODE, MyConstant.TOKEN_IS_NULL);
            }
        }

        if (!redisService.hasKey(token)) {
            throw new StudentException(MyConstant.TOKEN_CODE, MyConstant.TOKEN_DONT_EXISTENT);
        }

        boolean remove = redisService.remove(token);
        if (!remove) {
            throw new StudentException(MyConstant.TOKEN_CODE, MyConstant.TOKEN_DEL_FAIL);
        }
        return true;
    }

    @Override
    public boolean getAccessLimit(HttpServletRequest request, HttpServletResponse response, AccessLimit accessLimit) throws IOException {

        boolean temp = true;

        int seconds = accessLimit.seconds();
        int maxCount = accessLimit.maxCount();
        //从redis中获取用户访问的次数
        String ip = request.getHeader(MyConstant.X_FORWARDED_FOR);
        // 有可能ip是代理的
        if (ip == null || ip.length() == MyConstant.ZERO || MyConstant.IP_UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader(MyConstant.PROXY_CLIENT_IP);
        }
        if (ip == null || ip.length() == MyConstant.ZERO || MyConstant.IP_UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getHeader(MyConstant.WL_PROXY_CLIENT_IP);
        }
        if (ip == null || ip.length() == MyConstant.ZERO || MyConstant.IP_UNKNOWN.equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }

        Integer count = (Integer) redisService.get(MyConstant.SECOND_ACCESS + ip);
        if (count == null) {
            //第一次访问
            redisService.set(MyConstant.SECOND_ACCESS + ip, MyConstant.ONE, seconds);
        } else if (count < maxCount) {
            //加1
            count = count + MyConstant.ONE;
            redisService.incr(MyConstant.SECOND_ACCESS + ip, count);
        } else {
            //超出访问次数
            log.info("访问过快ip  ===> " + ip + " 且在   " + seconds + " 秒内超过最大限制  ===> " + maxCount + " 次数达到    ====> " + count);
            response.setCharacterEncoding(MyConstant.CHARACTER_ENCODING);
            response.setContentType(MyConstant.APPLICATION_JSON);

            ResultView result = ResultView.error(MyConstant.RETURN_MS);
            Object obj = JSONObject.toJSON(result);
            response.getWriter().write(JSONObject.toJSONString(obj));
            temp = false;
        }
        return temp;
    }


}
